Attribute VB_Name = "modEncodeQuery"
'' Character Encode Query for Passolo
'' (c) 2014 - 2016 by wanfu (Last modified on 2016.02.25)

'' Command Line Format: Command <String> <Switch>
'' Command: Name of this Macros file
'' String: String to be converted. If include Space characters, use "string" format
'' Switch:
'' -cp[N]: Codepage value. N is Numeric value of codepage. Such as: 936,1200 et.
'' -se: Escape text before convert text to code or after converted code to text. No this switch, not escape text
'' -et[N]: Escape type. N is escape type: 0 = Hex (Default), 1 = Hex escape, 2 = RUL escape, 3 = HTML escape
'' -ch: Every 2 Hex characters separated by spaces for Hex type
'' -ca: Convert ASCII characters to Hex escape or HTML escape. No this switch, not convert ASCII characters
'' -ci: Convert illegal character to URL escape. No this switch, not convert illegal character
'' -ac: Auto convert string to code or code to string after enter the data. No this switch, manually convert for UI mode only
'' -cs: Convert code to String. No this switch, Convert string to code
'' -noui: Do not display a user interface, run silently
'' Return: No
'' For example: modEncodeQuery.bas "This is strings.\r\nThis is converted Hex code." -cp:1201 -se -sc -ac -et:1
''              modEncodeQuery.bas "This is strings.\r\nThis is converted Hex code." cp:1201 se sc ac et:1

Option Explicit

Private Enum KnownCodePage
	CP_UNKNOWN = -1
	CP_ACP = 0
	CP_OEMCP = 1
	CP_MACCP = 2
	CP_THREAD_ACP = 3
	CP_SYMBOL = 42
' ARABIC
	CP_AWIN = 101 ' Bidi Windows codepage
	CP_709 = 102 ' MS-DOS Arabic Support 	CP 709
	CP_720 = 103 ' MS-DOS Arabic Support 	CP 720
	CP_A708 = 104 ' ASMO 708
	CP_A449 = 105 ' ASMO 449+
	CP_TARB = 106 ' MS Transparent Arabic
	CP_NAE = 107 ' Nafitha Enhanced Arabic Char Set
	CP_V4 = 108 ' Nafitha v 4.0
	CP_MA2 = 109 ' Mussaed Al Arabi (MA/2) 	CP 786
	CP_I864 = 110 ' IBM Arabic Supplement 	CP 864
	CP_A437 = 111 ' Ansi 437 codepage
	CP_AMAC = 112 ' Macintosh Code Page
' HEBREW
	CP_HWIN = 201 ' Bidi Windows codepage
	CP_862I = 202 ' IBM Hebrew Supplement 	CP 862
	CP_7BIT = 203 ' IBM Hebrew Supplement 	CP 862 Folded
	CP_ISO = 204 ' ISO Hebrew 8859-8 Character Set
	CP_H437 = 205 ' Ansi 437 codepage
	CP_HMAC = 206 ' Macintosh Code Page
' CODE PAGES
	CP_OEM_437 = 437
	CP_ARABICDOS = 708
	CP_DOS720 = 720
	CP_DOS737 = 737
	CP_DOS775 = 775
	CP_IBM850 = 850
	CP_IBM852 = 852
	CP_DOS861 = 861
	CP_DOS862 = 862
	CP_IBM866 = 866
	CP_DOS869 = 869
	CP_THAI = 874
	CP_EBCDIC = 875
	CP_JAPAN = 932
	CP_CHINA = 936
	CP_KOREA = 949
	CP_TAIWAN = 950
' UNICODE
	CP_UNICODELITTLE = 1200
	CP_UNICODEBIG = 1201
' CODE PAGES
	CP_EASTEUROPE = 1250
	CP_RUSSIAN = 1251
	CP_WESTEUROPE = 1252
	CP_GREEK = 1253
	CP_TURKISH = 1254
	CP_HEBREW = 1255
	CP_ARABIC = 1256
	CP_BALTIC = 1257
	CP_VIETNAMESE = 1258
' KOREAN
	CP_JOHAB = 1361
' MAC
	CP_MAC_ROMAN = 10000
	CP_MAC_JAPAN = 10001
	CP_MAC_ARABIC = 10004
	CP_MAC_GREEK = 10006
	CP_MAC_CYRILLIC = 10007
	CP_MAC_LATIN2 = 10029
	CP_MAC_TURKISH = 10081
' CODE PAGES
	CP_CHINESECNS = 20000
	CP_CHINESEETEN = 20002
	CP_IA5WEST = 20105
	CP_IA5GERMAN = 20106
	CP_IA5SWEDISH = 20107
	CP_IA5NORWEGIAN = 20108
	CP_ASCII = 20127
	CP_RUSSIANKOI8R = 20866
	CP_RUSSIANKOI8U = 21866
	CP_ISOLATIN1 = 28591
	CP_ISOEASTEUROPE = 28592
	CP_ISOTURKISH = 28593
	CP_ISOBALTIC = 28594
	CP_ISORUSSIAN = 28595
	CP_ISOARABIC = 28596
	CP_ISOGREEK = 28597
	CP_ISOHEBREW = 28598
	CP_ISOTURKISH2 = 28599
	CP_ISOLATIN9 = 28605
	CP_HEBREWLOG = 38598
	CP_USER = 50000
	CP_AUTOALL = 50001
	CP_JAPANNHK = 50220
	CP_JAPANESC = 50221
	CP_JAPANISO = 50222
	CP_KOREAISO = 50225
	CP_TAIWANISO = 50227
	CP_CHINAISO = 50229
	CP_AUTOJAPAN = 50932
	CP_AUTOCHINA = 50936
	CP_AUTOKOREA = 50949
	CP_AUTOTAIWAN = 50950
	CP_AUTORUSSIAN = 51251
	CP_AUTOGREEK = 51253
	CP_AUTOARABIC = 51256
	CP_JAPANEUC = 51932
	CP_CHINAEUC = 51936
	CP_KOREAEUC = 51949
	CP_TAIWANEUC = 51950
	CP_CHINAHZ = 52936
	CP_GB18030 = 54936
' UNICODE
	CP_UTF7 = 65000
	CP_UTF8 = 65001
End Enum

'ҳת
Private Declare Function MultiByteToWideChar Lib "kernel32.dll" ( _
	ByVal CodePage As Long, _
	ByVal dwFlags As Long, _
	ByRef lpMultiByteStr As Any, _
	ByVal cchMultiByte As Long, _
	ByVal lpWideCharStr As Long, _
	ByVal cchWideChar As Long) As Long
Private Declare Function WideCharToMultiByte Lib "kernel32.dll" ( _
	ByVal CodePage As Long, _
	ByVal dwFlags As Long, _
	ByVal lpWideCharStr As Long, _
	ByVal cchWideChar As Long, _
	ByRef lpMultiByteStr As Any, _
	ByVal cchMultiByte As Long, _
	ByVal lpDefaultChar As Long, _
	ByVal lpUsedDefaultChar As Long) As Long
Private Declare Function GetACP Lib "kernel32.dll" () As Long
'ڴ渴ƺͱȽϺ
Private Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" ( _
	ByRef Destination As Any, _
	ByRef Source As Any, _
	ByVal Length As Long)
'ıҶλ
Public Declare Function SendMessageLNG Lib "user32.dll" Alias "SendMessageA" ( _
	ByVal hwnd As Long, _
	ByVal wMsg As Long, _
	ByVal wParam As Long, _
	ByVal lParam As Long) As Long
'ڷؽؼľ
Private Declare Function GetFocus Lib "user32.dll" () As Long
'ڷؿؼIDľ
Private Declare Function GetDlgItem Lib "user32.dll" ( _
	ByVal hDlg As Long, _
	ByVal nIDDlgItem As Long) As Long

Private MsgList() As String,SetList() As String,RegExp As Object
Private Const Version = "2016.02.25"
Private Const Build = "160225"
Private Const TextJoinStr = vbCrLf
Private Const ItemJoinStr = ";"
Private Const EM_LIMITTEXT = &HC5	'ֵ,0 ñ༭ؼеı


' 
Sub Main
	' VBScript.RegExp Ƿ
	On Error Resume Next
	Set RegExp = CreateObject("VBScript.RegExp")
	If RegExp Is Nothing Then
		MsgBox(Err.Description & " - " & "VBScript.RegExp",vbInformation)
		Exit Sub
	End If
	On Error GoTo SysErrorMsg
	If GetMsgList(MsgList) = False Then GoTo SysErrorMsg
	SetList = SplitArgument(Command$,8)
	If StrToLong(SetList(7)) = 0 Then
		ConvertByUI(SetList)
	Else
		ConvertByNOUI(SetList)
	End If
	Exit Sub
	SysErrorMsg:
	If Err.Source <> "ExitSub" Then Call sysErrorMassage(Err,0)
End Sub


'зʽת
Public Function ConvertByNOUI(Settings() As String) As String
	If Settings(0) = "" Then Exit Function
	If StrToLong(Settings(6)) = 0 Then
		If StrToLong(Settings(2)) = 0 Then
			ConvertByNOUI = Settings(0)
		Else
			ConvertByNOUI = Convert(Settings(0))
		End If
		Select Case StrToLong(Settings(3))
		Case 0
			ConvertByNOUI = Byte2Hex(StringToByte(ConvertByNOUI,StrToLong(Settings(1))),0,-1)
		Case 1
			ConvertByNOUI = Str2HexEsc(ConvertByNOUI,StrToLong(Settings(1)),StrToLong(Settings(4)))
		Case 2
			ConvertByNOUI = Str2URLEsc(ConvertByNOUI,StrToLong(Settings(1)),StrToLong(Settings(4)))
		Case 3
			ConvertByNOUI = Str2HTMLEsc(ConvertByNOUI,StrToLong(Settings(1)),StrToLong(Settings(4)))
		End Select
	Else
		If StrToLong(Settings(4)) = 0 Then
			ConvertByNOUI = Replace$(Replace$(Settings(0)," ",""),vbCrLf,"")
		Else
			ConvertByNOUI = Settings(0)
		End If
		Select Case CheckHexStr(ConvertByNOUI,2,StrToLong(Settings(3)))
		Case 0
			Select Case StrToLong(Settings(3))
			Case 0
				ConvertByNOUI = ByteToString(HexStr2Bytes(ConvertByNOUI),StrToLong(Settings(1)))
			Case 1
				ConvertByNOUI = HexEsc2Str(ConvertByNOUI,StrToLong(Settings(1)))
			Case 2
				ConvertByNOUI = URLEsc2Str(ConvertByNOUI,StrToLong(Settings(1)))
			Case 3
				ConvertByNOUI = HTMLEsc2Str(ConvertByNOUI,StrToLong(Settings(1)))
			End Select
			If StrToLong(Settings(2)) = 1 Then
				ConvertByNOUI = ReConvert(ConvertByNOUI)
			End If
		Case 1
			PSL.Output MsgList(54)
		Case 2
			PSL.Output MsgList(55)
		Case 3
			PSL.Output MsgList(56)
		End Select
	End If
End Function


'Իʽ
Public Function ConvertByUI(Settings() As String) As String
	Begin Dialog UserDialog 780,350,Replace$(Replace$(MsgList(37),"%v",Version),"%b",Build),.ConvertByUIDlgFunc ' %GRID:10,7,1,1
		CheckBox 690,7,30,14,"",.CheckBox
		PushButton 680,0,90,21,"",.TempButton
		TextBox 0,3,110,21,.StrLengthBox
		TextBox 0,3,110,21,.CodeLengthBox
		TextBox 0,0,0,21,.SuppValueBox

		Text 10,7,130,14,MsgList(38),.StrText
		CheckBox 400,7,270,14,MsgList(39),.SeparatHexBox
		CheckBox 400,7,270,14,MsgList(40),.HEXASCIICharBox
		CheckBox 400,7,270,14,MsgList(41),.URLLegalCharBox
		CheckBox 400,7,270,14,MsgList(40),.HTMLASCIICharBox
		TextBox 10,28,660,119,.StrBox,1
		PushButton 680,28,90,28,MsgList(42),.Str2CodeButton
		PushButton 680,56,90,28,MsgList(43),.StrCopyButton
		PushButton 680,84,90,28,MsgList(44),.StrPasteButton
		PushButton 680,112,90,28,MsgList(45),.StrCleanButton

		Text 10,154,130,14,MsgList(46),.CodeText
		DropListBox 150,4,230,21,MsgList(),.ModeList
		DropListBox 150,151,230,21,MsgList(),.CPNameList
		DropListBox 150,151,230,21,MsgList(),.CPValueList
		CheckBox 400,154,130,14,MsgList(47),.ConvertBox
		CheckBox 540,154,130,14,MsgList(48),.ModeBox
		TextBox 10,175,660,168,.CodeBox,1
		PushButton 680,175,90,28,MsgList(49),.Code2StrButton
		PushButton 680,203,90,28,MsgList(50),.CodeCopyButton
		PushButton 680,231,90,28,MsgList(51),.CodePasteButton
		PushButton 680,259,90,28,MsgList(52),.CodeCleanButton

		PushButton 680,315,90,28,MsgList(53),.AboutButton
		CancelButton 730,7,30,14,.CancelButton
	End Dialog
	Dim dlg As UserDialog
	If Dialog(dlg) = 0 Then Exit Function
	If StrToLong(Settings(6)) = 0 Then
		ConvertByUI = dlg.CodeBox
	Else
		ConvertByUI = dlg.StrBox
	End If
End Function


'Ի
Private Function ConvertByUIDlgFunc(DlgItem$, Action%, SuppValue&) As Boolean
	Dim i As Long,Temp As String
	DlgValue "CheckBox",0
	Select Case Action%
	Case 1 ' Ի򴰿ڳʼ
		DlgVisible "CancelButton",False
		DlgVisible "CPValueList",False
		DlgVisible "CheckBox",False
		DlgVisible "TempButton",False
		DlgVisible "StrLengthBox",False
		DlgVisible "CodeLengthBox",False
		DlgVisible "SuppValueBox",False
		DlgValue "CheckBox",0
		DlgText "SuppValueBox",CStr$(SuppValue)

		ReDim TempList(21) As String
		TempList(0) = CStr$(CP_OEMCP)			'ϵͳĬ
		TempList(1) = CStr$(CP_MACCP)
		TempList(2) = CStr$(CP_THREAD_ACP)
		TempList(3) = CStr$(CP_WESTEUROPE)		' 1 (ANSI) = 1252
		TempList(4) = CStr$(CP_EASTEUROPE)		' 2 (ŷ) = 1250
		TempList(5) = CStr$(CP_RUSSIAN)			' (˹) = 1251
		TempList(6) = CStr$(CP_GREEK)			'ϣ = 1253
		TempList(7) = CStr$(CP_TURKISH)			' 5 () = 1254
		TempList(8) = CStr$(CP_HEBREW)			'ϣ = 1255
		TempList(9) = CStr$(CP_ARABIC)			' = 1256
		TempList(10) = CStr$(CP_BALTIC)			'޵ĺ = 1257
		TempList(11) = CStr$(CP_VIETNAMESE)		'Խ = 1258
		TempList(12) = CStr$(CP_JAPAN)			' = 932
		TempList(13) = CStr$(CP_CHINA)			' = 936
		TempList(14) = CStr$(CP_GB18030)		' = 54936
		TempList(15) = CStr$(CP_KOREA)			' = 949
		TempList(16) = CStr$(CP_TAIWAN) 		' = 950
		TempList(17) = CStr$(CP_THAI)			'̩ = 874
		TempList(18) = CStr$(CP_UNICODELITTLE)	'UnicodeLE = 1200
		TempList(19) = CStr$(CP_UNICODEBIG)		'UnicodeBE = 1201
		TempList(20) = CStr$(CP_UTF7)			'UTF-7 = 65000
		TempList(21) = CStr$(CP_UTF8)			'UTF-8 = 65001
		DlgListBoxArray "CPValueList",TempList()
		For i = 0 To 21
			TempList(i) = TempList(i) & " - " & MsgList(i + 14)
		Next i
		DlgListBoxArray "CPNameList",TempList()
		TempList = Split(MsgList(36),ItemJoinStr)
		DlgListBoxArray "ModeList",TempList()

		If StrToLong(SetList(6)) = 0 Then
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(SuppValue,DlgControlId("StrBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),Len(SetList(0)),False)
			DlgText "StrLengthBox",CStr$(i)
			DlgText "StrBox",SetList(0)
		Else
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(SuppValue,DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),Len(SetList(0)),False)
			DlgText "CodeLengthBox",CStr$(i)
			DlgText "CodeBox",SetList(0)
		End If
		DlgText "CPValueList",SetList(1)
		DlgValue "ConvertBox",StrToLong(SetList(2))
		DlgValue "ModeList",StrToLong(SetList(3))
		Select Case StrToLong(SetList(3))
		Case 0
			DlgValue "SeparatHexBox",StrToLong(SetList(4))
		Case 1
			DlgValue "HEXASCIICharBox",StrToLong(SetList(4))
		Case 2
			DlgValue "URLLegalCharBox",StrToLong(SetList(4))
		Case 3
			DlgValue "HTMLASCIICharBox",StrToLong(SetList(4))
		End Select
		DlgValue "ModeBox",StrToLong(SetList(5))

		If DlgText("CPValueList") = "" Then
			DlgText "CPValueList",CStr$(GetACP)
		End If
		DlgValue "CPNameList",DlgValue("CPValueList")

		Select Case DlgValue("ModeList")
		Case 0
			DlgVisible "HEXASCIICharBox",False
			DlgVisible "URLLegalCharBox",False
			DlgVisible "HTMLASCIICharBox",False
		Case 1
			DlgVisible "SeparatHexBox",False
			DlgVisible "URLLegalCharBox",False
			DlgVisible "HTMLASCIICharBox",False
		Case 2
			DlgVisible "SeparatHexBox",False
			DlgVisible "HEXASCIICharBox",False
			DlgVisible "HTMLASCIICharBox",False
		Case 3
			DlgVisible "SeparatHexBox",False
			DlgVisible "HEXASCIICharBox",False
			DlgVisible "URLLegalCharBox",False
		End Select

		If DlgText("StrBox") <> "" Then
			If DlgValue("ConvertBox") = 0 Then
				Temp = DlgText("StrBox")
			Else
				Temp = Convert(DlgText("StrBox"))
			End If
			i = Len(Temp)
			DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
			DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			If DlgValue("ModeBox") = 0 Then Exit Function
			Select Case DlgValue("ModeList")
			Case 0
				Temp = Byte2Hex(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
				If DlgValue("SeparatHexBox") = 1 Then Temp = SeparatHex(Temp)
			Case 1
				Temp = Str2HexEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HEXASCIICharBox"))
			Case 2
				Temp = Str2URLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("URLLegalCharBox"))
			Case 3
				Temp = Str2HTMLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HTMLASCIICharBox"))
			End Select
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(SuppValue,DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),Len(Temp),False)
			DlgText "CodeLengthBox",CStr$(i)
			DlgText "CodeBox",Temp
		ElseIf DlgText("CodeBox") <> "" Then
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgValue("ModeList") = 0 Then
				Temp = Replace$(Replace$(DlgText("CodeBox")," ",""),vbCrLf,"")
			Else
				Temp = DlgText("CodeBox")
			End If
			Select Case CheckHexStr(Temp,2,DlgValue("ModeList"))
			Case 0
				Select Case DlgValue("ModeList")
				Case 0
					Temp = ByteToString(HexStr2Bytes(Temp),CLng(Trim$(DlgText("CPValueList"))))
				Case 1
					Temp = HexEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 2
					Temp = URLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 3
					Temp = HTMLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				End Select
				i = Len(Temp)
				DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				If DlgValue("ConvertBox") = 1 Then Temp = ReConvert(Temp)
				'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
				i = GetDlgItem(SuppValue,DlgControlId("StrBox"))
				i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),Len(Temp),False)
				DlgText "StrLengthBox",CStr$(i)
				DlgText "StrBox",Temp
			Case 1
				MsgBox(MsgList(54),vbOkOnly+vbInformation,MsgList(0))
			Case 2
				MsgBox(MsgList(55),vbOkOnly+vbInformation,MsgList(0))
			Case 3
				MsgBox(MsgList(56),vbOkOnly+vbInformation,MsgList(0))
			End Select
		Else
			DlgText "StrText",Replace$(Replace$(MsgList(38),"%s","0"),"%d","00")
			DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s","0"),"%d","00")
		End If
	Case 2 ' ֵĻ߰˰ť
		If DlgItem$ = "CancelButton" Then Exit Function
		ConvertByUIDlgFunc = True ' ֹ°ťرնԻ򴰿
		Select Case DlgItem$
		Case "AboutButton"
			MsgBox Replace$(Replace$(MsgList(12),"%v",Version),"%b",Build),vbOkOnly+vbInformation,MsgList(13)
		Case "StrCopyButton"
			Clipboard DlgText("StrBox")
		Case "CodeCopyButton"
			Clipboard DlgText("CodeBox")
		Case "StrPasteButton"
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("StrBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),Len(Clipboard),True)
			DlgText "StrLengthBox",CStr$(i)
			DlgText "StrBox",Clipboard
		Case "CodePasteButton"
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),Len(Clipboard),True)
			DlgText "CodeLengthBox",CStr$(i)
			DlgText "CodeBox",Clipboard
		Case "StrCleanButton"
			DlgText "StrBox",""
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("StrBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),0,True)
			DlgText "StrLengthBox",CStr$(i)
		Case "CodeCleanButton"
			DlgText "CodeBox",""
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),0,True)
			DlgText "CodeLengthBox",CStr$(i)
		Case "CPNameList"
			DlgValue "CPValueList",DlgValue("CPNameList")
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgText("StrBox") <> "" Then
				DlgItem$ = "Str2CodeButton"
			ElseIf DlgText("CodeBox") <> ""Then
				DlgItem$ = "Code2StrButton"
			End If
		Case "URLCPNameList"
			DlgValue "URLCPValueList",DlgValue("URLCPNameList")
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgText("StrBox") <> "" Then
				DlgItem$ = "Str2CodeButton"
			ElseIf DlgText("CodeBox") <> ""Then
				DlgItem$ = "Code2StrButton"
			End If
		Case "HTMLCPNameList"
			DlgValue "HTMLCPValueList",DlgValue("HTMLCPNameList")
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgText("StrBox") <> "" Then
				DlgItem$ = "Str2CodeButton"
			ElseIf DlgText("CodeBox") <> ""Then
				DlgItem$ = "Code2StrButton"
			End If
		Case "ModeList"
			Select Case DlgValue("ModeList")
			Case 0
				DlgVisible "SeparatHexBox",True
				DlgVisible "HEXASCIICharBox",False
				DlgVisible "URLLegalCharBox",False
				DlgVisible "HTMLASCIICharBox",False
			Case 1
				DlgVisible "SeparatHexBox",False
				DlgVisible "HEXASCIICharBox",True
				DlgVisible "URLLegalCharBox",False
				DlgVisible "HTMLASCIICharBox",False
			Case 2
				DlgVisible "SeparatHexBox",False
				DlgVisible "HEXASCIICharBox",False
				DlgVisible "URLLegalCharBox",True
				DlgVisible "HTMLASCIICharBox",False
			Case 3
				DlgVisible "SeparatHexBox",False
				DlgVisible "HEXASCIICharBox",False
				DlgVisible "URLLegalCharBox",False
				DlgVisible "HTMLASCIICharBox",True
			End Select
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgText("StrBox") <> "" Then
				DlgItem$ = "Str2CodeButton"
			ElseIf DlgText("CodeBox") <> ""Then
				DlgItem$ = "Code2StrButton"
			End If
		Case "SeparatHexBox"
			If InStr(DlgText("CodeBox")," ") Then
				DlgText "CodeBox",Join$(Split(DlgText("CodeBox")," "),"")
			Else
				DlgText "CodeBox",SeparatHex(DlgText("CodeBox"))
			End If
			DlgValue "CheckBox",1
		Case "HEXASCIICharBox","URLLegalCharBox","HTMLASCIICharBox"
			If DlgValue("ModeBox") = 1 Then DlgItem$ = "Str2CodeButton"
		Case "ConvertBox"
			If DlgText("StrBox") = "" Then Exit Function
			If DlgValue("ConvertBox") = 0 Then
				Temp = DlgText("StrBox")
			Else
				Temp = Convert(DlgText("StrBox"))
			End If
			i = Len(Temp)
			DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
			DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			If DlgValue("ModeBox") = 1 Then DlgItem$ = "Str2CodeButton"
		Case "ModeBox"
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgText("StrBox") <> "" Then
				DlgItem$ = "Str2CodeButton"
			ElseIf DlgText("CodeBox") <> ""Then
				DlgItem$ = "Code2StrButton"
			End If
		End Select
		Select Case DlgItem$
		Case "Str2CodeButton"
			If DlgText("StrBox") = "" Then Exit Function
			If Temp = "" Then
				If DlgValue("ConvertBox") = 0 Then
					Temp = DlgText("StrBox")
				Else
					Temp = Convert(DlgText("StrBox"))
				End If
				i = Len(Temp)
				DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			End If
			Select Case DlgValue("ModeList")
			Case 0
				Temp = Byte2Hex(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
				If DlgValue("SeparatHexBox") = 1 Then Temp = SeparatHex(Temp)
			Case 1
				Temp = Str2HexEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HEXASCIICharBox"))
			Case 2
				Temp = Str2URLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("URLLegalCharBox"))
			Case 3
				Temp = Str2HTMLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HTMLASCIICharBox"))
			End Select
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),Len(Temp),True)
			DlgText "CodeLengthBox",CStr$(i)
			DlgText "CodeBox",Temp
			DlgValue "CheckBox",1
		Case "Code2StrButton"
			If DlgText("CodeBox") = "" Then Exit Function
			If DlgValue("ModeList") = 0 Then
				Temp = Replace$(Replace$(DlgText("CodeBox")," ",""),vbCrLf,"")
			Else
				Temp = DlgText("CodeBox")
			End If
			Select Case CheckHexStr(Temp,2,DlgValue("ModeList"))
			Case 0
				Select Case DlgValue("ModeList")
				Case 0
					Temp = ByteToString(HexStr2Bytes(Temp),CLng(Trim$(DlgText("CPValueList"))))
				Case 1
					Temp = HexEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 2
					Temp = URLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 3
					Temp = HTMLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				End Select
				i = Len(Temp)
				DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				If DlgValue("ConvertBox") = 1 Then Temp = ReConvert(Temp)
				'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
				i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("StrBox"))
				i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),Len(Temp),True)
				DlgText "StrLengthBox",CStr$(i)
				DlgText "StrBox",Temp
				DlgValue "CheckBox",1
			Case 1
				MsgBox(MsgList(54),vbOkOnly+vbInformation,MsgList(0))
			Case 2
				MsgBox(MsgList(55),vbOkOnly+vbInformation,MsgList(0))
			Case 3
				MsgBox(MsgList(56),vbOkOnly+vbInformation,MsgList(0))
			End Select
		End Select
	Case 3 ' ıϿı
		If DlgValue("CheckBox") = 1 Then Exit Function
		Select Case DlgItem$
		Case "StrBox"
			If DlgText("StrBox") = "" Then
				DlgText "StrText",Replace$(Replace$(MsgList(38),"%s","0"),"%d","00")
				'DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s","0"),"%d","00")
				'DlgText "CodeBox",""
				Exit Function
			End If
			If DlgValue("ConvertBox") = 0 Then
				Temp = DlgText("StrBox")
			Else
				Temp = Convert(DlgText("StrBox"))
			End If
			i = Len(Temp)
			DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
			DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			If DlgValue("ModeBox") = 0 Then Exit Function
			Select Case DlgValue("ModeList")
			Case 0
				Temp = Byte2Hex(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
				If DlgValue("SeparatHexBox") = 1 Then Temp = SeparatHex(Temp)
			Case 1
				Temp = Str2HexEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HEXASCIICharBox"))
			Case 2
				Temp = Str2URLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("URLLegalCharBox"))
			Case 3
				Temp = Str2HTMLEsc(Temp,CLng(Trim$(DlgText("CPValueList"))),DlgValue("HTMLASCIICharBox"))
			End Select
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),Len(Temp),True)
			DlgText "CodeLengthBox",CStr$(i)
			DlgText "CodeBox",Temp
			DlgValue "CheckBox",1
		Case "CodeBox"
			If DlgText("CodeBox") = "" Then
				'DlgText "StrText",Replace$(Replace$(MsgList(38),"%s","0"),"%d","00")
				DlgText "CodeText",Replace$(Replace$(MsgList(45),"%s","0"),"%d","00")
				'DlgText "StrBox",""
				Exit Function
			End If
			If DlgValue("ModeBox") = 0 Then Exit Function
			If DlgValue("ModeList") = 0 Then
				Temp = Replace$(Replace$(DlgText("CodeBox")," ",""),vbCrLf,"")
			Else
				Temp = DlgText("CodeBox")
			End If
			Select Case CheckHexStr(Temp,2,DlgValue("ModeList"))
			Case 0
				Select Case DlgValue("ModeList")
				Case 0
					Temp = ByteToString(HexStr2Bytes(Temp),CLng(Trim$(DlgText("CPValueList"))))
				Case 1
					Temp = HexEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 2
					Temp = URLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				Case 3
					Temp = HTMLEsc2Str(Temp,CLng(Trim$(DlgText("CPValueList"))))
				End Select
				i = Len(Temp)
				DlgText "StrText",Replace$(Replace$(MsgList(38),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				DlgText "CodeText",Replace$(Replace$(MsgList(46),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
				If DlgValue("ConvertBox") = 1 Then Temp = ReConvert(Temp)
				'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
				i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("StrBox"))
				i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),Len(Temp),True)
				DlgText "StrLengthBox",CStr$(i)
				DlgText "StrBox",Temp
				DlgValue "CheckBox",1
			Case 1
				MsgBox(MsgList(54),vbOkOnly+vbInformation,MsgList(0))
			Case 2
				MsgBox(MsgList(55),vbOkOnly+vbInformation,MsgList(0))
			Case 3
				MsgBox(MsgList(56),vbOkOnly+vbInformation,MsgList(0))
			End Select
		End Select
	Case 4 ' 㱻
		Select Case DlgItem$
		Case "StrBox"
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("StrBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("StrLengthBox")),StrToLong(DlgText("StrLengthBox")) + Len(Clipboard),False)
			DlgText "StrLengthBox",CStr$(i)
		Case "CodeBox"
			'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
			i = GetDlgItem(CLng(DlgText("SuppValueBox")),DlgControlId("CodeBox"))
			i = SetTextBoxLength(i,StrToLong(DlgText("CodeLengthBox")),StrToLong(DlgText("CodeLengthBox")) + Len(Clipboard),False)
			DlgText "CodeLengthBox",CStr$(i)
		End Select
	End Select
End Function


'õĴҳUTF-8:65001GB2312:936GB1803054936UTF-765000
Private Function MultiByteToUTF16(UTF8() As Byte, ByVal CodePage As Long) As String
	Dim bufSize As Long,lRet As Long
	'bufSize = MultiByteToWideChar(CodePage, 0&, VarPtr(UTF8(0)), UBound(UTF8) + 1, 0, 0)
	bufSize = MultiByteToWideChar(CodePage, 0&, UTF8(0), UBound(UTF8) + 1, 0, 0)
	'If CodePage = CP_UTF8 Then bufSize = (UBound(UTF8) + 1) * 2 Else bufSize = UBound(UTF8) + 1
	MultiByteToUTF16 = Space(bufSize)
	'lRet = MultiByteToWideChar(CodePage, 0&, VarPtr(UTF8(0)), UBound(UTF8) + 1, StrPtr(MultiByteToUTF16), bufSize)
	lRet = MultiByteToWideChar(CodePage, 0&, UTF8(0), UBound(UTF8)+1, StrPtr(MultiByteToUTF16), bufSize)
	If lRet > 0 Then
		MultiByteToUTF16 = Left$(MultiByteToUTF16, lRet)
	End If
End Function


Private Function UTF16ToMultiByte(ByVal UTF16 As String, ByVal CodePage As Long) As Byte()
	Dim bufSize As Long,lRet As Long
	ReDim arr(0) As Byte
	bufSize = WideCharToMultiByte(CodePage, 0&, StrPtr(UTF16), Len(UTF16), arr(0), 0, 0, 0)
	'If CodePage = CP_UTF8 Then bufSize = 2 * LenB(UTF16) - 1 Else bufSize = LenB(UTF16)
	If bufSize < 1 Then bufSize = 1
	ReDim arr(bufSize - 1) As Byte
	lRet = WideCharToMultiByte(CodePage, 0&, StrPtr(UTF16), Len(UTF16), arr(0), bufSize, 0, 0)
	If lRet > 0 Then
		ReDim Preserve arr(lRet - 1) As Byte
	End If
	UTF16ToMultiByte = arr
End Function


'ֽת Hex 
'StartPos <= EndPos ȡλλ Hex 룬ȡλλ Hex 
Private Function Byte2Hex(Bytes() As Byte,ByVal StartPos As Long,ByVal EndPos As Long) As String
	Dim i As Long,n As Long
	If StartPos < 0 Then StartPos = LBound(Bytes)
	If EndPos < 0 Then EndPos = UBound(Bytes)
	Byte2Hex = Space$((Abs(EndPos - StartPos) + 1) * 2)
	n = 1
	For i = StartPos To EndPos Step IIf(StartPos <= EndPos,1,-1)
		Mid$(Byte2Hex,n,2) = Right$("0" & Hex$(Bytes(i)),2)
		n = n + 2
	Next i
End Function


'ֽת Hex ת
'StartPos <= EndPos ȡλλ Hex 룬ȡλλ Hex 
Private Function Byte2HexEsc(Bytes() As Byte,ByVal StartPos As Long,ByVal EndPos As Long,ByVal CodePage As Long) As String
	Dim i As Long,n As Long
	If StartPos < 0 Then StartPos = LBound(Bytes)
	If EndPos < 0 Then EndPos = UBound(Bytes)
	Select Case CodePage
	Case CP_UNICODELITTLE
		Byte2HexEsc = Space$((Abs(EndPos - StartPos) + 1) * 3)
		n = 1
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			Mid$(Byte2HexEsc,n,6) = "\u" & Right$("0" & Hex$(Bytes(i + 1)),2) & Right$("0" & Hex$(Bytes(i)),2)
			n = n + 6
		Next i
	Case CP_UNICODEBIG
		Byte2HexEsc = Space$((Abs(EndPos - StartPos) + 1) * 3)
		n = 1
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			Mid$(Byte2HexEsc,n,6) = "\u" & Right$("0" & Hex$(Bytes(i)),2) & Right$("0" & Hex$(Bytes(i + 1)),2)
			n = n + 6
		Next i
	Case Else
		Byte2HexEsc = Space$((Abs(EndPos - StartPos) + 1) * 4)
		n = 1
		For i = StartPos To EndPos Step IIf(StartPos <= EndPos,1,-1)
			Mid$(Byte2HexEsc,n,4) = "\x" & Right$("0" & Hex$(Bytes(i)),2)
			n = n + 4
		Next i
	End Select
End Function


'ֽת RUL ת
'StartPos <= EndPos ȡλλ Hex 룬ȡλλ Hex 
Private Function Byte2URLEsc(Bytes() As Byte,ByVal StartPos As Long,ByVal EndPos As Long,ByVal CodePage As Long) As String
	Dim i As Long,n As Long
	If StartPos < 0 Then StartPos = LBound(Bytes)
	If EndPos < 0 Then EndPos = UBound(Bytes)
	Select Case CodePage
	Case CP_UNICODELITTLE
		Byte2URLEsc = Space$((Abs(EndPos - StartPos) + 1) * 3)
		n = 1
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			Mid$(Byte2URLEsc,n,6) = "%u" & Right$("0" & Hex$(Bytes(i + 1)),2) & Right$("0" & Hex$(Bytes(i)),2)
			n = n + 6
		Next i
	Case CP_UNICODEBIG
		Byte2URLEsc = Space$((Abs(EndPos - StartPos) + 1) * 3)
		n = 1
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			Mid$(Byte2URLEsc,n,6) = "%u" & Right$("0" & Hex$(Bytes(i)),2) & Right$("0" & Hex$(Bytes(i + 1)),2)
			n = n + 6
		Next i
	Case Else
		Byte2URLEsc = Space$((Abs(EndPos - StartPos) + 1) * 3)
		n = 1
		For i = StartPos To EndPos Step IIf(StartPos <= EndPos,1,-1)
			Mid$(Byte2URLEsc,n,3) = "%" & Right$("0" & Hex$(Bytes(i)),2)
			n = n + 3
		Next i
	End Select
End Function


'ֽת HTML ת(תֽ붼 UnicodeLE)
'StartPos <= EndPos ȡλλ Hex 룬ȡλλ Hex 
Private Function Byte2HTMLEsc(Bytes() As Byte,ByVal StartPos As Long,ByVal EndPos As Long,ByVal CodePage As Long) As String
	Dim i As Long,n As Long,TempList() As String
	If StartPos < 0 Then StartPos = LBound(Bytes)
	If EndPos < 0 Then EndPos = UBound(Bytes)
	Select Case CodePage
 	Case CP_UNICODELITTLE
		ReDim TempList(Abs(EndPos - StartPos)) As String
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			TempList(n) = "&#" & CStr$(Bytes(i) + 256& * Bytes(i + 1)) & ";"
			n = n + 1
		Next i
		Byte2HTMLEsc = Join$(TempList,"")
	Case CP_UNICODEBIG
		ReDim TempList(Abs(EndPos - StartPos)) As String
		For i = StartPos To EndPos - 1 Step IIf(StartPos <= EndPos,2,-2)
			TempList(n) = "&#" & CStr$(Bytes(i + 1) + 256& * Bytes(i)) & ";"
			n = n + 1
		Next i
		Byte2HTMLEsc = Join$(TempList,"")
	Case Else
		ReDim TempList(Abs(EndPos - StartPos)) As String
		For i = StartPos To EndPos Step IIf(StartPos <= EndPos,1,-1)
			TempList(n) = "&#" & CStr$(Bytes(i) + 0) & ";"
			n = n + 1
		Next i
		Byte2HTMLEsc = Join$(TempList,"")
	End Select
End Function


'ת HEX Ϊֽ
Private Function HexStr2Bytes(ByVal HexStr As String) As Byte()
	Dim i As Long,n As Long,Length As Long
	Length = Len(HexStr)
	If Length > 1 Then
		ReDim tempByte(Length \ 2 - 1) As Byte
		For i = 1 To Length Step 2
			tempByte(n) = Val("&H" & Mid$(HexStr,i,2))
			n = n + 1
		Next i
	Else
		ReDim tempByte(0) As Byte
	End If
	HexStr2Bytes = tempByte
End Function


'ַתֽ
Private Function StringToByte(ByVal textStr As String,ByVal CodePage As Long) As Byte()
	If textStr = "" Then
		ReDim StringToByte(0) As Byte
		Exit Function
	End If
	Select Case CodePage
	Case CP_UNICODELITTLE
		StringToByte = textStr
	Case CP_UNICODEBIG
		StringToByte = textStr
		StringToByte = LowByte2HighByte(StringToByte)
	Case Else
		StringToByte = UTF16ToMultiByte(textStr,CodePage)	'ָҳתUnicodeַΪANSI
		'StringToByte = StrConv$(textStr,vbFromUnicode)		'ҳתUnicodeַΪANSI
	End Select
End Function


'ֽתַ
Private Function ByteToString(Bytes() As Byte,ByVal CodePage As Long) As String
	Select Case CodePage
	Case CP_UNICODELITTLE
		ByteToString = Bytes
	Case CP_UNICODEBIG
		ByteToString = LowByte2HighByte(Bytes)
	Case Else
		ByteToString = MultiByteToUTF16(Bytes,CodePage)
	End Select
End Function


'ÿֽڵĸֽں͵ֽڻ UNICODE LITTLE  UNICODE BIG ֽ໥ת
Private Function LowByte2HighByte(Bytes() As Byte) As Byte()
	Dim i As Long,Temp() As Byte
	Temp = Bytes
	For i = LBound(Temp) To UBound(Temp) - 1 Step 2
		Temp(i) = Bytes(i + 1)
		Temp(i + 1) = Bytes(i)
	Next i
	LowByte2HighByte = Temp
End Function


'תֽ飬ֵֽĸֽں͵ֽڻ
Private Function ReverseValByte(Bytes() As Byte,ByVal StartPos As Long,ByVal EndPos As Long) As Byte()
	Dim i As Long,Temp() As Byte
	If StartPos < 0 Then StartPos = LBound(Bytes)
	If EndPos < 0 Then EndPos = UBound(Bytes)
	Temp = Bytes
	For i = StartPos To EndPos
		Temp(i) = Bytes(EndPos - i)
	Next i
	ReverseValByte = Temp
End Function


'ִǷҪ
Private Function CheckHexStr(ByVal textStr As String,ByVal Length As Integer,ByVal Mode As Integer) As Long
	Dim i As Long,Temp As String,Matches As Object
	If textStr = "" Then Exit Function
	Select Case Mode
	Case 0
		If (Len(textStr) Mod Length) <> 0 Then
			CheckHexStr = 1
			Exit Function
		End If
		If CheckStr(textStr,"[0-9A-Fa-f]",0,1) = False Then
			CheckHexStr = 3
		End If
		Exit Function
	Case 1
		textStr = LCase$(textStr)
		Do
			With RegExp
				.Global = True
				.IgnoreCase = False
				.Pattern = IIf(Length = 2,"(\\x","(\\u") & "[0-9A-Fa-f]{1,Length})+"
				Set Matches = .Execute(textStr)
			End With
			If Matches.Count > 0 Then
				For i = 0 To Matches.Count - 1
					Temp = Replace$(Replace$(Matches(i).Value,"\x",""),"\u","")
					If (Len(Temp) Mod Length) <> 0 Then
						CheckHexStr = 1
						Exit Function
					End If
					If CheckStr(Temp,"[0-9A-Fa-f]",0,1) = False Then
						CheckHexStr = 3
						Exit Function
					End If
				Next i
			End If
			Length = Length * 2
		Loop Until Length > 4
	Case 2
		textStr = LCase$(textStr)
		With RegExp
			.Global = True
			.IgnoreCase = False
			.Pattern = "(%(u|)[0-9A-Fa-f]{1,Length})+"
			Set Matches = .Execute(textStr)
		End With
		If Matches.Count > 0 Then
			For i = 0 To Matches.Count - 1
				Temp = Replace$(Replace$(Matches(i).Value,"%u",""),"%","")
				If (Len(Temp) Mod Length) <> 0 Then
					CheckHexStr = 1
					Exit Function
				End If
				If CheckStr(Temp,"[0-9A-Fa-f]",0,1) = False Then
					CheckHexStr = 3
					Exit Function
				End If
			Next i
		End If
	Case 3
		With RegExp
			.Global = True
			.IgnoreCase = False
			.Pattern = "(&#[0-9]{1,};)"
			Set Matches = RegExp.Execute(textStr)
		End With
		If Matches.Count > 0 Then
			For i = 0 To Matches.Count - 1
				Temp = Replace$(Replace$(Matches(i).Value,"&#",""),";","")
				If Len(Temp) < 2 Or Len(Temp) > 5 Then
					CheckHexStr = 2
					Exit Function
				End If
				If CheckStr(Temp,"[0-9]",0,1) = False Then
					CheckHexStr = 3
					Exit Function
				End If
			Next i
		End If
	End Select
End Function


'ִת
Private Function Convert(ByVal ConverString As String) As String
	Convert = ConverString
	If Convert = "" Then Exit Function
	If InStr(Convert,"\") = 0 Then Exit Function
	If InStr(Convert,"\\") Then Convert = Replace$(Convert,"\\","*a!N!d*")
	If InStr(Convert,"\r\n") Then Convert = Replace$(Convert,"\r\n",vbCrLf)
	If InStr(Convert,"\r\n") Then Convert = Replace$(Convert,"\r\n",vbNewLine)
	If InStr(Convert,"\r") Then Convert = Replace$(Convert,"\r",vbCr)
	If InStr(Convert,"\r") Then Convert = Replace$(Convert,"\r",vbNewLine)
	If InStr(Convert,"\n") Then Convert = Replace$(Convert,"\n",vbLf)
	If InStr(Convert,"\b") Then Convert = Replace$(Convert,"\b",vbBack)
	If InStr(Convert,"\f") Then Convert = Replace$(Convert,"\f",vbFormFeed)
	If InStr(Convert,"\v") Then Convert = Replace$(Convert,"\v",vbVerticalTab)
	If InStr(Convert,"\t") Then Convert = Replace$(Convert,"\t",vbTab)
	If InStr(Convert,"\'") Then Convert = Replace$(Convert,"\'","'")
	If InStr(Convert,"\""") Then Convert = Replace$(Convert,"\""","""")
	If InStr(Convert,"\?") Then Convert = Replace$(Convert,"\?","?")
	If InStr(Convert,"\") Then Convert = ConvertB(Convert)
	If InStr(Convert,"\0") Then Convert = Replace$(Convert,"\0",vbNullChar)
	If InStr(Convert,"*a!N!d*") Then Convert = Replace$(Convert,"*a!N!d*","\")
End Function


'ת˽ƻʮת
Private Function ConvertB(ByVal ConverString As String) As String
	Dim i As Long,j As Long,EscStr As String,ConvCode As String
	ConvertB = ConverString
	i = InStr(ConvertB,"\")
	Do While i > 0
		EscStr = Mid$(ConvertB,i,2)
		Select Case EscStr
		Case "\x", "\X"
			ConvCode = Mid$(ConvertB,i + 2,2)
			If CheckStr(UCase$(ConvCode),"[0-9A-Fa-f]",0,1) = True Then
				j = Val("&H" & ConvCode)
				ConvertB = Replace$(ConvertB,EscStr & ConvCode,Val2Bytes(j,2,0))
			End If
		Case "\u", "\U"
			ConvCode = Mid$(ConvertB,i + 2,4)
			If CheckStr(UCase$(ConvCode),"[0-9A-Fa-f]",0,1) = True Then
				j = Val("&H" & ConvCode)
				ConvertB = Replace$(ConvertB,EscStr & ConvCode,Val2Bytes(j,2,0))
			End If
		Case Is <> ""
			EscStr = "\"
			For j = 3 To 1 Step -1
				ConvCode = Mid$(ConvertB,i + 1,j)
				If CheckStr(ConvCode,"[0-7]",0,1) = True Then
					j = Val("&O" & ConvCode)
					If j > 256 Then
						ConvCode = Left$(ConvCode,2)
						j = Val("&O" & ConvCode)
					End If
					ConvertB = Replace$(ConvertB,EscStr & ConvCode,Val2Bytes(j,2,0))
					Exit For
				End If
			Next j
		End Select
		i = InStr(i + 1,ConvertB,"\")
	Loop
End Function


'ִת
Private Function ReConvert(ByVal ConverString As String) As String
	ReConvert = ConverString
	If ReConvert = "" Then Exit Function
	If InStr(ReConvert,"\") Then ReConvert = Replace$(ReConvert,"\","\\")
	If InStr(ReConvert,vbCrLf) Then ReConvert = Replace$(ReConvert,vbCrLf,"\r\n")
	If InStr(ReConvert,vbNewLine) Then ReConvert = Replace$(ReConvert,vbNewLine,"\r\n")
	If InStr(ReConvert,vbCr) Then ReConvert = Replace$(ReConvert,vbCr,"\r")
	If InStr(ReConvert,vbLf) Then ReConvert = Replace$(ReConvert,vbLf,"\n")
	If InStr(ReConvert,vbBack) Then ReConvert = Replace$(ReConvert,vbBack,"\b")
	If InStr(ReConvert,vbFormFeed) Then ReConvert = Replace$(ReConvert,vbFormFeed,"\f")
	If InStr(ReConvert,vbVerticalTab) Then ReConvert = Replace$(ReConvert,vbVerticalTab,"\v")
	If InStr(ReConvert,vbTab) Then ReConvert = Replace$(ReConvert,vbTab,"\t")
	If InStr(ReConvert,vbNullChar) Then ReConvert = Replace$(ReConvert,vbNullChar,"\0")
	ReConvert = ReConvertB(ReConvert)
End Function


'תչַΪʮת
Private Function ReConvertB(ByVal ConverString As String) As String
	Dim i As Long,Dec As Long,Temp As String
	ReConvertB = ConverString
	For i = 1 To Len(ConverString)
		Temp = Mid$(ConverString,i,1)
		If InStr(ReConvertB,Temp) Then
			Dec = AscW(Temp)
			If (Dec > 0 And Dec < 32) Or (Dec > 126 And Dec < 256) Then
				ReConvertB = Replace$(ReConvertB,Temp,"\x" & Right$("0" & Hex$(Dec),2))
			End If
		End If
	Next i
End Function


'ִǷַָ(ıͨȽ)
'Mode = 0 ִǷַָҳַָλ
'Mode = 1 ִǷַָֻ
'Mode = 2 ִǷַָ
'Mode = 3 ִǷֻСдַָʱ IgnoreCase Ч
'Mode = 4 ִǷַͬStrNum Ϊַ
'StrRange  ִ鷶Χ ( [Min - Max|Min - Max] ʾΧ)
Private Function CheckStr(ByVal textStr As String,ByVal StrRange As String,Optional ByVal StrNum As Long, _
				Optional ByVal Mode As Long,Optional ByVal IgnoreCase As Boolean) As Long
	Dim i As Long,Temp As String
	If StrRange = "" Then Exit Function
	If Trim$(textStr) = "" Then Exit Function
	Select Case Mode
	Case 0
		If IgnoreCase = True Then
			textStr = LCase$(textStr)
			StrRange = LCase$(StrRange)
		End If
		StrRange = Replace$(StrRange,"]|[","")
		If (textStr Like "*" & StrRange & "*") = False Then Exit Function
		For i = 1 To Len(textStr)
			If (Mid$(textStr,i,1) Like StrRange) = True Then
				CheckStr = i
				Exit For
			End If
		Next i
	Case 1
		If IgnoreCase = True Then
			textStr = LCase$(textStr)
			StrRange = LCase$(StrRange)
		End If
		If (textStr Like "*[!" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = False Then
			CheckStr = True
		End If
	Case 2
		If IgnoreCase = True Then
			textStr = LCase$(textStr)
			StrRange = LCase$(StrRange)
		End If
		If (textStr Like "*[" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = True Then
			CheckStr = True
		End If
	Case 3
		If InStr(textStr," ") Then Exit Function
		If Len(textStr) < 2 Then Exit Function
		If LCase$(textStr) = textStr Then Exit Function
		If UCase$(textStr) = textStr Then Exit Function
		If (textStr Like "*[!" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = True Then Exit Function
		If (Mid$(textStr,2) Like "*" & Split(StrRange,"|")(0) & "*") = True Then
			If (Mid$(textStr,2) Like "*" & Split(StrRange,"|")(1) & "*") = True Then
				CheckStr = True
			End If
		End If
	Case 4
		If StrNum < 2 Then Exit Function
		If Len(textStr) < StrNum Then Exit Function
		If InStr(textStr," ") Then Exit Function
		If IsNumeric(textStr) = True Then Exit Function
		If IsDate(textStr) = True Then Exit Function
		If IgnoreCase = True Then
			textStr = LCase$(textStr)
			StrRange = LCase$(StrRange)
		End If
		If (textStr Like "*" & Replace$(StrRange,"]|[","") & "*") = False Then Exit Function
		For i = 1 To Len(textStr)
			Temp = Mid$(textStr,i,1)
			If IsNumeric(Temp) = False Then
				If Temp <> "." Then
					If InStr(textStr,String$(StrNum,Temp)) Then
						CheckStr = True
						Exit For
					End If
				End If
			End If
		Next i
	End Select
End Function


'ַַʮֽڳ
'Mode = 0 ת, ת
Private Function StrHexLength(ByVal textStr As String,ByVal CodePage As Long,ByVal Mode As Long) As Long
	If textStr = "" Then Exit Function
	If Mode = 0 Then textStr = Convert(textStr)
	If CodePage = CP_UNICODELITTLE Or CodePage = CP_UNICODEBIG Then
		'Bin = textStr
		'StrHexLength = UBound(Bin) + 1
		StrHexLength = LenB(textStr)
	Else
		ReDim Bin(0) As Byte
		StrHexLength = WideCharToMultiByte(CodePage, 0&, StrPtr(textStr), Len(textStr), Bin(0), 0, 0, 0)
	End If
End Function


'в
'' Command [String] [Codepage] [Sring escape] [Escape type] [Multibyte only] [Convert mode] [Convert type] [UI mode]
'' Command Line Format: Command <String> <Switch>
'' Command: Name of this Macros file
'' String: String to be converted. If include Space characters, use "string" format
'' Switch:
'' -cp[N]: Codepage value. N is Numeric value of codepage. Such as: 936,1200 et.
'' -se: Escape text before convert text to code or after converted code to text. No this switch, not escape text
'' -et[N]: Escape type. N is escape type: 0 = Hex (Default), 1 = Hex escape, 2 = RUL escape, 3 = HTML escape
'' -ch: Every 2 Hex characters separated by spaces for Hex type
'' -ca: Convert ASCII characters to Hex escape or HTML escape. No this switch, not convert ASCII characters
'' -ci: Convert illegal character to URL escape. No this switch, not convert illegal character
'' -ac: Auto convert string to code or code to string after enter the data. No this switch, manually convert for UI mode only
'' -cs: Convert code to String. No this switch, Convert string to code
'' -noui: Do not display a user interface, run silently
'' Return: No
'' For example: modEncodeQuery.bas "This is strings.\r\nThis is converted Hex code." -cp:1201 -se -sc -ac -et:1
''              modEncodeQuery.bas "This is strings.\r\nThis is converted Hex code." cp:1201 se sc ac et:1
Private Function SplitArgument(ByVal Argument As String,ByVal MaxNum As Long) As String()
	Dim i As Long,k As Long,Temp As String,TempList() As String
	ReDim TempArray(MaxNum - 1) As String
	If Argument = "" Then
		SplitArgument = TempArray
		Exit Function
	End If
	'ӺǰֵС
	TempList = Split(Argument)
	k = UBound(TempList)
	MaxNum = IIf(k - MaxNum > 0,k - MaxNum,1)
	For i = k To MaxNum Step -1
		Temp = LCase$(Trim$(TempList(i)))
		If CheckStr(Temp,"-cp:[0-9]",0,1) = True Then
			Temp = "-cp:"
		ElseIf CheckStr(Temp,"cp:[0-9]",0,1) = True Then
			Temp = "-cp:"
		ElseIf CheckStr(Temp,"-et:[0-3]",0,1) = True Then
			Temp = "-et:"
		ElseIf CheckStr(Temp,"et:[0-3]",0,1) = True Then
			Temp = "-et:"
		End If
		Select Case Temp
		Case "-cp:"
			If TempArray(1) = "" Then
				TempArray(1) = Mid$(Trim$(TempList(i)),5)
				TempList(i) = ""
			End If
		Case "-se","se"
			If TempArray(2) = "" Then
				TempArray(2) = "1"
				TempList(i) = ""
			End If
		Case "-et:"
			If TempArray(3) = "" Then
				TempArray(3) = Mid$(Trim$(TempList(i)),5)
				TempList(i) = ""
			End If
		Case "-ch","-ca","-ci","ch","ca","ci"
			If TempArray(4) = "" Then
				TempArray(4) = "1"
				TempList(i) = ""
			End If
		Case "-ac","ac"
			If TempArray(5) = "" Then
				TempArray(5) = "1"
				TempList(i) = ""
			End If
		Case "-cs","cs"
			If TempArray(6) = "" Then
				TempArray(6) = "1"
				TempList(i) = ""
			End If
		Case "-noui","noui"
			If TempArray(7) = "" Then
				TempArray(7) = "1"
				TempList(i) = ""
			End If
		End Select
	Next i
	TempArray(0) = Join$(TempList," ")
	'ɾıǰ˫
	If Left$(LTrim$(TempArray(0)),1) = """" And Right$(RTrim$(TempArray(0)),1) = """" Then
		TempArray(0) = RemoveBackslash(TempArray(0),"""","""",2)
	End If
	SplitArgument = TempArray
End Function


'ȥִǰָ PreStr  AppStr
'fType = -1 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 0 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 1 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 2 ȥִǰĿոָ PreStr  AppStr 1 Σȥִǰո
'fType > 2 ȥִǰĿոָ PreStr  AppStr 1 Σȥִǰո
Private Function RemoveBackslash(ByVal Path As String,ByVal PreStr As String,ByVal AppStr As String,ByVal fType As Long) As String
	Dim i As Long,a As Long,p As Long,Stemp As Boolean
	RemoveBackslash = Path
	If Path = "" Then Exit Function
	a = Len(AppStr)
	p = Len(PreStr)
	If fType > -1 Then RemoveBackslash = Trim(RemoveBackslash)
	Do
		Stemp = False
		If p <> 0 Then
			If Left$(RemoveBackslash,p) = PreStr Then
				RemoveBackslash = Mid$(RemoveBackslash,p + 1)
				Stemp = True
			End If
		End If
		If a <> 0 Then
			If Right$(RemoveBackslash,a) = AppStr Then
				RemoveBackslash = Left$(RemoveBackslash,Len(RemoveBackslash) - a)
				Stemp = True
			End If
		End If
		If fType = 1 Or fType > 2 Then RemoveBackslash = Trim$(RemoveBackslash)
		If Stemp = True Then
			If fType < 2 Then i = 0 Else i = 1
		Else
			i = 1
		End If
	Loop Until i = 1
End Function


'תַΪֵ
Private Function StrToLong(ByVal mStr As String,Optional ByVal DefaultValue As Long) As Long
	On Error GoTo errHandle
	StrToLong = CLng(mStr)
	Exit Function
	errHandle:
	StrToLong = DefaultValue
End Function


'ִǷΪգǿշ True
Private Function CheckArray(DataList() As String) As Boolean
	Dim i As Long
	On Error GoTo errHandle
	For i = LBound(DataList) To UBound(DataList)
		If DataList(i) <> "" Then
			CheckArray = True
			Exit For
		End If
	Next i
	errHandle:
End Function


'ʽ HEX ִ
Private Function FormatHexStr(ByVal textStr As String,ByVal Length As Integer) As String
	If textStr = "" Then Exit Function
	If (Len(textStr) Mod Length) = 0 Then
		FormatHexStr = textStr
		Exit Function
	End If
	FormatHexStr = "0" & textStr
End Function


'תֵΪֽ(ڳȵĸλض)
Private Function Val2Bytes(ByVal Value As Long,ByVal Length As Integer,ByVal CodePage As Long) As Byte()
	On Error GoTo errHandle
	ReDim Bytes(Length - 1) As Byte
	CopyMemory Bytes(0), Value, Length
	If CodePage = CP_UNICODEBIG Then
		Val2Bytes = ReverseValByte(Bytes,0,-1)
	Else
		Val2Bytes = Bytes
	End If
	Exit Function
	errHandle:
	ReDim Bytes(0) As Byte
	Val2Bytes = Bytes
End Function


'ÿַոָ
Private Function SeparatHex(ByVal textStr As String) As String
	Dim i As Long,j As Long,n As Long
	j = Len(textStr)
	SeparatHex = Space$(j * 2)
	n = 1
	For i = 1 To j - 1 Step 2
		Mid$(SeparatHex,n,2) = Mid(textStr,i,2) & " "
		n = n + 3
	Next i
	SeparatHex = Trim$(SeparatHex)
End Function


'ַתתģʽ
Private Function Str2HexEsc(ByVal textStr As String,ByVal CodePage As Long,ByVal MultibyteOnly As Long) As String
	If textStr = "" Then Exit Function
	If MultibyteOnly = 0 Then
		Str2HexEsc = Byte2HexEsc(StringToByte(textStr,CodePage),0,-1,CodePage)
		Exit Function
	End If
	Dim i As Long,Matches As Object
	Str2HexEsc = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = False
	RegExp.Pattern = "[^\x00-\x7F]+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count = 0 Then Exit Function
	For i = Matches.Count - 1 To 0 Step -1
		With Matches(i)
			If .FirstIndex > 0 Then
				Str2HexEsc = Left$(Str2HexEsc,.FirstIndex) & _
							Byte2HexEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2HexEsc,.FirstIndex + .Length + 1)
			Else
				Str2HexEsc = Byte2HexEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2HexEsc,.Length + 1)
			End If
		End With
	Next i
End Function


'Hex תתַ
Private Function HexEsc2Str(ByVal textStr As String,ByVal CodePage As Long) As String
	Dim i As Long,Temp As String,Matches As Object
	If textStr = "" Then Exit Function
	HexEsc2Str = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = True
	RegExp.Pattern = "(\\u[0-9A-Fa-f]{4})+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count > 0 Then
		For i = 0 To Matches.Count - 1
			Temp = Replace$(LCase$(Matches(i).Value),"\u","")
			If CodePage = CP_UNICODELITTLE Then
				Temp = ByteToString(LowByte2HighByte(HexStr2Bytes(Temp)),CodePage)
			Else
				Temp = ByteToString(HexStr2Bytes(Temp),CodePage)
			End If
			HexEsc2Str = Replace$(HexEsc2Str,Matches(i).Value,Temp,,1)
		Next i
	Else
		RegExp.Pattern = "(\\x[0-9A-Fa-f]{2})+"
		Set Matches = RegExp.Execute(textStr)
		If Matches.Count = 0 Then Exit Function
		For i = 0 To Matches.Count - 1
			Temp = Replace$(LCase$(Matches(i).Value),"\x","")
			Temp = ByteToString(HexStr2Bytes(Temp),CodePage)
			HexEsc2Str = Replace$(HexEsc2Str,Matches(i).Value,Temp,,1)
		Next i
	End If
End Function


'ַת URL ת
Private Function Str2URLEsc(ByVal textStr As String,ByVal CodePage As Long,ByVal MultibyteOnly As Long) As String
	If textStr = "" Then Exit Function
	If MultibyteOnly = 0 Then
		Str2URLEsc = Byte2URLEsc(StringToByte(textStr,CodePage),0,-1,CodePage)
		Exit Function
	End If
	Dim i As Long,Matches As Object
	Str2URLEsc = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = False
	'RegExp.Pattern = "[\x00-x20\x22\x25\x3C\x3E\x5B-\x5E\x60\x7B-\x7D\x7F]+"
	RegExp.Pattern = "[^\x21\x23\x24\x26-\x3B\x3D\x3F-\x5A\x5F\x61-\x7A\x7E]+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count = 0 Then Exit Function
	For i = Matches.Count - 1 To 0 Step -1
		With Matches(i)
			If .FirstIndex > 0 Then
				Str2URLEsc = Left$(Str2URLEsc,.FirstIndex) & _
							Byte2URLEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2URLEsc,.FirstIndex + .Length + 1)
			Else
				Str2URLEsc = Byte2URLEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2URLEsc,.Length + 1)
			End If
		End With
	Next i
End Function


'URL תתַ
Private Function URLEsc2Str(ByVal textStr As String,ByVal CodePage As Long) As String
	Dim i As Long,Temp As String,Matches As Object
	If textStr = "" Then Exit Function
	URLEsc2Str = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = True
	RegExp.Pattern = "(%u[0-9A-Fa-f]{4})+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count > 0 Then
		For i = 0 To Matches.Count - 1
			Temp = Replace$(LCase$(Matches(i).Value),"%u","")
			If CodePage = CP_UNICODELITTLE Then
				Temp = ByteToString(LowByte2HighByte(HexStr2Bytes(Temp)),CodePage)
			Else
				Temp = ByteToString(HexStr2Bytes(Temp),CodePage)
			End If
			URLEsc2Str = Replace$(URLEsc2Str,Matches(i).Value,Temp,,1)
		Next i
	Else
		RegExp.Pattern = "(%[0-9A-Fa-f]{2})+"
		Set Matches = RegExp.Execute(textStr)
		If Matches.Count = 0 Then Exit Function
		For i = 0 To Matches.Count - 1
			Temp = Replace$(Matches(i).Value,"%","")
			Temp = ByteToString(HexStr2Bytes(Temp),CodePage)
			URLEsc2Str = Replace$(URLEsc2Str,Matches(i).Value,Temp,,1)
		Next i
	End If
End Function


'ַת HTML ת
Private Function Str2HTMLEsc(ByVal textStr As String,ByVal CodePage As Long,ByVal MultibyteOnly As Long) As String
	If textStr = "" Then Exit Function
	If MultibyteOnly = 0 Then
		Str2HTMLEsc = Byte2HTMLEsc(StringToByte(textStr,CodePage),0,-1,CodePage)
		Exit Function
	End If
	Dim i As Long,Matches As Object
	Str2HTMLEsc = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = False
	RegExp.Pattern = "[^\x00-\x7F]+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count = 0 Then Exit Function
	For i = Matches.Count - 1 To 0 Step -1
		With Matches(i)
			If .FirstIndex > 0 Then
				Str2HTMLEsc = Left$(Str2HTMLEsc,.FirstIndex) & _
							Byte2HTMLEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2HTMLEsc,.FirstIndex + .Length + 1)
			Else
				Str2HTMLEsc = Byte2HTMLEsc(StringToByte(.Value,CodePage),0,-1,CodePage) & _
							Mid$(Str2HTMLEsc,.Length + 1)
			End If
		End With
	Next i
End Function


'HTML תתַ
Private Function HTMLEsc2Str(ByVal textStr As String,ByVal CodePage As Long) As String
	Dim i As Long,j As Long,n As Long,TempList() As String,Matches As Object
	If textStr = "" Then Exit Function
	HTMLEsc2Str = textStr
	RegExp.Global = True
	RegExp.IgnoreCase = False
	RegExp.Pattern = "(&#[0-9]{2,5};)+"
	Set Matches = RegExp.Execute(textStr)
	If Matches.Count = 0 Then Exit Function
	If CodePage = CP_UNICODEBIG Then CodePage = CP_UNICODELITTLE
	n = IIf(CodePage = CP_UNICODELITTLE,2,1)
	For i = 0 To Matches.Count - 1
		TempList = Split(Replace$(Matches(i).Value,"&#",""),";")
		For j = 0 To UBound(TempList) - 1
			TempList(j) = Byte2Hex(Val2Bytes(CLng(TempList(j)),n,0),0,-1)
		Next j
		TempList(0) = ByteToString(HexStr2Bytes(Join$(TempList,"")),CodePage)
		HTMLEsc2Str = Replace$(HTMLEsc2Str,Matches(i).Value,TempList(0),,1)
	Next i
End Function


'ñ༭ؼеıȣԭ󳤶Ϊ30000ַ˫ַֽ1
Private Function SetTextBoxLength(ByVal hwnd As Long,ByVal OldLength As Long,ByVal NewLength As Long,ByVal Mode As Boolean) As Long
	SetTextBoxLength = OldLength
	If NewLength < OldLength Then
		If Mode = False Then Exit Function
		If NewLength <= 25000 Then NewLength = 25000
	End If
	SetTextBoxLength = NewLength
	SendMessageLNG(hwnd,EM_LIMITTEXT,NewLength + 5000,0&)
End Function


'Ϣ
Private Sub sysErrorMassage(sysError As ErrObject,ByVal fType As Long)
	Dim TempArray() As String,MsgList() As String
	Dim ErrorNumber As Long,ErrorSource As String,ErrorDescription As String
	Dim TitleMsg As String,ContinueMsg As String,Msg As String

	ErrorNumber = sysError.Number
	ErrorSource = sysError.Source
	ErrorDescription = sysError.Description

	TitleMsg = "Error"
	Select Case fType
	Case 0
		ContinueMsg = vbCrLf & vbCrLf & "The program cannot continue and will exit."
	Case 1
		ContinueMsg = vbCrLf & vbCrLf & "Do you want to continue?"
	Case 2
		ContinueMsg = vbCrLf & vbCrLf & "The program will continue to run."
	End Select

	If CheckArray(MsgList) = True Then
		TitleMsg = MsgList(0)
		Select Case fType
		Case 0
			ContinueMsg = MsgList(1)
		Case 1
			ContinueMsg = MsgList(2)
		Case 2
			ContinueMsg = MsgList(3)
		End Select

		Select Case ErrorSource
		Case ""
			If ErrorNumber = 10051 And PSL.Version >= 1500 Then
				Msg = Replace$(MsgList(4),"%s",ErrorSource)
			Else
				Msg = Replace$(Replace$(MsgList(5),"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
			End If
		Case "NotSection"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(Replace$(MsgList(6),"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotValue"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(Replace$(MsgList(7),"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotReadFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotUnWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotOpenFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotINIFile"
			Msg = Replace$(MsgList(8),"%s",ErrorDescription)
		Case "NotExitFile"
			Msg = Replace$(MsgList(9),"%s",ErrorDescription)
		Case "NotVersion"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(MsgList(10),"%s",TempArray(0))
			Msg = Replace$(Replace$(Msg,"%d",TempArray(1)),"%v",TempArray(2))
		Case Else
			Msg = Replace$(MsgList(11),"%s",ErrorSource)
			Msg = Replace$(Replace$(Msg,"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
		End Select
	Else
		Select Case ErrorSource
		Case ""
			If ErrorNumber = 10051 And PSL.Version >= 1500 Then
				Msg = "Unable to open the file. Please verify the file path and file name" & _
						"contains characters in Asian languages. " & vbCrLf & _
						"Note: Passolo 2015 Version of the macro engine does not recognize" & _
						"the file path and file name contains Asian language characters."
			Else
				Msg = "An Error occurred in the program design." & vbCrLf & "Error Code: %d, Content: %v" & _
						vbCrLf & "Please restart the Passolo try and please report to the software developer."
				Msg = Replace$(Replace$(Msg,"%s",CStr(ErrorNumber)),"%v",ErrorDescription)
			End If
		Case "NotSection"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file is missing [%s] section." & vbCrLf & "%d"
			Msg = Replace$(Replace$(Msg,"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotValue"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file is missing [%s] Value." & vbCrLf & "%d"
			Msg = Replace$(Replace$(Msg,"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotReadFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotUnWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotOpenFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotINIFile"
			Msg = "The following contents of the file is not correct." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",ErrorDescription)
		Case "NotExitFile"
			Msg = "The following file does not exist! Please check and try again." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",ErrorDescription)
		Case "NotVersion"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file version is %d, requires version at least %v." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",TempArray(0))
			Msg = Replace$(Replace$(Msg,"%d",TempArray(1)),"%v",TempArray(2))
		Case Else
			Msg = "Your system is missing %s server." & vbCrLf & "Error Code: %d, Content: %v"
			Msg = Replace$(Msg,"%s",ErrorSource)
			Msg = Replace$(Replace$(Msg,"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
		End Select
	End If

	If Msg = "" Then Exit Sub
	Select Case fType
	Case 0
		MsgBox(Msg & ContinueMsg,vbOkOnly+vbInformation,TitleMsg)
		Exit All
	Case 1
		If MsgBox(Msg & ContinueMsg,vbYesNo+vbInformation,TitleMsg) = vbNo Then
			Exit All
		End If
	Case Else
		MsgBox(Msg & ContinueMsg,vbOkOnly+vbInformation,TitleMsg)
	End Select
End Sub


'Ϣַ
Private Function GetMsgList(MsgList() As String) As Boolean
	Dim i As Integer
	ReDim MsgList(56) As String
	On Error GoTo errHandle
	MsgList(0) = "Error"
	MsgList(1) = "\r\n\r\nThe program cannot continue and will exit."
	MsgList(2) = "\r\n\r\nDo you want to continue?"
	MsgList(3) = "\r\n\r\nThe program will continue to run."
	MsgList(4) = "Unable to open the file. Please verify the file path and file name" & _
				"contains characters in Asian languages.\r\n" & _
				"Note: Passolo 2015 Version of the macro engine does not recognize" & _
				"the file path and file name contains Asian language characters."
	MsgList(5) = "An Error occurred in the program design.\r\nError Code: %d, Content: %v\r\n" & _
				"Please restart the Passolo try and please report to the software developer."
	MsgList(6) = "The following file is missing [%s] section.\r\n%d"
	MsgList(7) = "The following file is missing [%s] Value.\r\n%d"
	MsgList(8) = "The following contents of the file is not correct.\r\n%s"
	MsgList(9) = "The following file does not exist! Please check and try again.\r\n%s"
	MsgList(10) = "The following file version is %d, requires version at least %v.\r\n%s"
	MsgList(11) = "Your system is missing %s server.\r\nError Code: %d, Content: %v"

	MsgList(12) = "Version: %v (Build %b)\r\n" & _
				"OS Version: Windows XP/2000 or higher\r\n" & _
				"Passolo Version: Passolo 5.0 or higher\r\n" & _
				"Copyright: Hanzify\r\nLicense: Freeware\r\n" & _
				"HomePage: http://www.hanzify.org\r\n" & _
				"Author: Hanzify member - wanfu (2014 - 2016)\r\n" & _
				"E-mail: z_shangyi@163.com"
	MsgList(13) = "About Character Encode Query"

	MsgList(14) = "OEM"
	MsgList(15) = "MAC"
	MsgList(16) = "THREAD"
	MsgList(17) = "WEST EUROPE"
	MsgList(18) = "EAST EUROPE"
	MsgList(19) = "RUSSIAN"
	MsgList(20) = "GREEK"
	MsgList(21) = "TURKISH"
	MsgList(22) = "HEBREW"
	MsgList(23) = "ARABIC"
	MsgList(24) = "BALTIC"
	MsgList(25) = "VIETNAMESE"
	MsgList(26) = "JAPAN"
	MsgList(27) = "CHINA GB2312"
	MsgList(28) = "CHINA GB18030"
	MsgList(29) = "KOREA"
	MsgList(30) = "TAIWAN"
	MsgList(31) = "THAI"
	MsgList(32) = "Unicode LE"
	MsgList(33) = "Unicode BE"
	MsgList(34) = "UTF-7"
	MsgList(35) = "UTF-8"

	MsgList(36) = "Hex;Hex Escape;URL Escape;HTML Escape"

	MsgList(37) = "Character Encode Query - Version %v (Build %b)"
	MsgList(38) = "Text (%s/%d):"
	MsgList(39) = "Byte Mode Output
	MsgList(40) = "Multibyte Characters Only"
	MsgList(41) = "Unsafe Characters Only"
	MsgList(42) = "To Code"
	MsgList(43) = "Copy"
	MsgList(44) = "Paste"
	MsgList(45) = "Clear"
	MsgList(46) = "Code (%s/%d):"
	MsgList(47) = "Escape Text"
	MsgList(48) = "Auto Convert"
	MsgList(49) = "To Text"
	MsgList(50) = "Copy"
	MsgList(51) = "Paste"
	MsgList(52) = "Clear"
	MsgList(53) = "About"

	MsgList(54) = "Number of Hex code errors."
	MsgList(55) = "Number of digit code errors."
	MsgList(56) = "Illegal character in Hex code."
	For i = 0 To UBound(MsgList)
		If InStr(MsgList(i),"\\") Then MsgList(i) = Replace$(MsgList(i),"\\","*a!N!d*")
		If InStr(MsgList(i),"\r\n") Then MsgList(i) = Replace$(MsgList(i),"\r\n",vbCrLf)
		If InStr(MsgList(i),"\t") Then MsgList(i) = Replace$(MsgList(i),"\t",vbTab)
		If InStr(MsgList(i),"*a!N!d*") Then MsgList(i) = Replace$(MsgList(i),"*a!N!d*","\")
	Next i
	GetMsgList = True
	Exit Function
	errHandle:
	ReDim MsgList(0) As String
End Function
